@@ -108,10 +108,6 @@ progress_callback(void *data, double total_to_download, double now_downloaded)
         if (result == Py_None) {
             // Assume that None means that everything is ok
             ret = LR_CB_OK;
-#if PY_MAJOR_VERSION < 3
-        } else if (PyInt_Check(result)) {
-            ret = PyInt_AS_LONG(result);
-#endif
         } else if (PyLong_Check(result)) {
             ret = (int) PyLong_AsLong(result);
         } else {
@@ -209,10 +205,6 @@ hmf_callback(void *data, const char *msg, const char *url, const char *metadata)
         if (result == Py_None) {
             // Assume that None means that everything is ok
             ret = LR_CB_OK;
-#if PY_MAJOR_VERSION < 3
-        } else if (PyInt_Check(result)) {
-            ret = PyInt_AS_LONG(result);
-#endif
         } else if (PyLong_Check(result)) {
             ret = (int) PyLong_AsLong(result);
         } else {
@@ -301,6 +293,8 @@ py_setopt(_HandleObject *self, PyObject *args)
     case LRO_MIRRORLISTURL:
     case LRO_METALINKURL:
     case LRO_USERPWD:
+    case LRO_USERNAME:
+    case LRO_PASSWORD:
     case LRO_PROXY:
     case LRO_PROXYUSERPWD:
     case LRO_DESTDIR:
@@ -310,6 +304,9 @@ py_setopt(_HandleObject *self, PyObject *args)
     case LRO_SSLCLIENTCERT:
     case LRO_SSLCLIENTKEY:
     case LRO_SSLCACERT:
+    case LRO_PROXY_SSLCLIENTCERT:
+    case LRO_PROXY_SSLCLIENTKEY:
+    case LRO_PROXY_SSLCACERT:
     case LRO_CACHEDIR:
     {
         char *str = NULL, *alloced = NULL;
@@ -373,7 +370,10 @@ py_setopt(_HandleObject *self, PyObject *args)
     case LRO_FETCHMIRRORS:
     case LRO_FASTESTMIRROR:
     case LRO_SSLVERIFYPEER:
+    case LRO_SSLVERIFYSTATUS:
     case LRO_SSLVERIFYHOST:
+    case LRO_PROXY_SSLVERIFYPEER:
+    case LRO_PROXY_SSLVERIFYHOST:
     case LRO_ADAPTIVEMIRRORSORTING:
     case LRO_FTPUSEEPSV:
     case LRO_PRESERVETIME:
@@ -382,8 +382,8 @@ py_setopt(_HandleObject *self, PyObject *args)
         long d;
 
         // Default values for None attribute
-        if (obj == Py_None && (option == LRO_SSLVERIFYPEER ||
-                               option == LRO_SSLVERIFYHOST))
+        if (obj == Py_None && (option == LRO_SSLVERIFYPEER || option == LRO_PROXY_SSLVERIFYPEER ||
+                               option == LRO_SSLVERIFYHOST || option == LRO_PROXY_SSLVERIFYHOST))
         {
             d = 1;
         } else if (obj == Py_None && option == LRO_ADAPTIVEMIRRORSORTING) {
@@ -423,10 +423,6 @@ py_setopt(_HandleObject *self, PyObject *args)
 
         if (PyLong_Check(obj))
             d = PyLong_AsLong(obj);
-#if PY_MAJOR_VERSION < 3
-        else if (PyInt_Check(obj))
-            d = PyInt_AS_LONG(obj);
-#endif
         else if (obj == Py_None) {
             // None stands for default value
             switch (option) {
@@ -481,10 +477,6 @@ py_setopt(_HandleObject *self, PyObject *args)
 
         if (PyLong_Check(obj))
             d = PyLong_AsLong(obj);
-#if PY_MAJOR_VERSION < 3
-        else if (PyInt_Check(obj))
-            d = PyInt_AS_LONG(obj);
-#endif
         else if (obj == Py_None) {
             /* Default options */
             if (option == LRO_PROXYPORT)
@@ -524,10 +516,6 @@ py_setopt(_HandleObject *self, PyObject *args)
 
         if (PyLong_Check(obj))
             d = (gint64) PyLong_AsLongLong(obj);
-#if PY_MAJOR_VERSION < 3
-        else if (PyInt_Check(obj))
-            d = (gint64) PyInt_AS_LONG(obj);
-#endif
         else if (obj == Py_None) {
             /* Default options */
             if (option == LRO_MAXSPEED)
@@ -878,6 +866,9 @@ py_getinfo(_HandleObject *self, PyObject *args)
     case LRI_SSLCLIENTCERT:
     case LRI_SSLCLIENTKEY:
     case LRI_SSLCACERT:
+    case LRI_PROXY_SSLCLIENTCERT:
+    case LRI_PROXY_SSLCLIENTKEY:
+    case LRI_PROXY_SSLCACERT:
     case LRI_CACHEDIR:
         res = lr_handle_getinfo(self->handle,
                                 &tmp_err,
@@ -906,7 +897,10 @@ py_getinfo(_HandleObject *self, PyObject *args)
     case LRI_FASTESTMIRROR:
     case LRI_FASTESTMIRRORMAXAGE:
     case LRI_SSLVERIFYPEER:
+    case LRI_SSLVERIFYSTATUS:
     case LRI_SSLVERIFYHOST:
+    case LRI_PROXY_SSLVERIFYPEER:
+    case LRI_PROXY_SSLVERIFYHOST:
     case LRI_ALLOWEDMIRRORFAILURES:
     case LRI_ADAPTIVEMIRRORSORTING:
     case LRI_OFFLINE:
