@@ -6,6 +6,7 @@ import hashlib
 import unittest
 import tempfile
 import xattr
+import errno
 
 import tests.servermock.yum_mock.config as config
 
@@ -758,7 +759,7 @@ class TestCaseYumPackagesDownloading(TestCaseWithServer):
                            "user.librepo.downloadinprogress".encode("utf-8"),
                            "".encode("utf-8"))
         except IOError as err:
-            if err.errno == 95:
+            if err.errno == errno.EOPNOTSUPP:
                 self.skipTest('extended attributes are not supported')
             raise
 
